// Server.h (Test Server)

#pragma once

#using "System.ServiceModel.dll"

namespace TestServer
{
using namespace System;
using namespace System::Threading;
using namespace System::ServiceModel;
using namespace System::Collections::Generic;

private enum class ClientMirrorStatus { Active, Detached, TimedOut };

private ref class ClientMirrorImage
{
public:
  ClientMirrorImage(int nInstanceNumber, Timers::Timer ^tmrHeartbeat);

  void AssertActive(bool bResetTimeout);
  void Detach();
  bool IsActive();

  virtual String ^ToString() override;

private:
  void HeartbeatHandler(Object ^sender, Timers::ElapsedEventArgs ^e);
  void DetachFromHeartbeat();

private:
  initonly int m_nInstanceNumber;
  Timers::Timer ^m_tmrHeartbeat;
  int m_nTimeout;
  ClientMirrorStatus m_status;
};

[ServiceContract(SessionMode=SessionMode::Required)]
public interface class ITestServer
{
  [OperationContract]
  int GetClientInstanceNumber();

  [OperationContract]
  String ^GetSharedData(int nInstanceNumber);

  [OperationContract]
  void SetSharedData(int nInstanceNumber, String ^strData);

  [OperationContract]
  void Detach(int nInstanceNumber);
};

[ServiceBehavior(InstanceContextMode=InstanceContextMode::Single)]
public ref class Server : public ITestServer
{
public:
  Server();
  ~Server();

  // Service contract interface method implementation prototypes

  virtual int GetClientInstanceNumber();
  virtual String ^GetSharedData(int nInstanceNumber);
  virtual void SetSharedData(int nInstanceNumber, String ^strData);
  virtual void Detach(int nInstanceNumber);

internal:
  static bool Startup(Action ^dltShutdownRequestCallback, Action<int> ^dltStatusReportCallback);
  static void Shutdown();
  static int GetServerObjectCount();
  static array<String ^> ^GetInstanceInfo();
  static bool IsPrimaryServerInstanceIdle();

private:
  bool IsIdle();
  array<String ^> ^GetInstanceInfoCore();
  void HeartbeatHandler(Object ^sender, Timers::ElapsedEventArgs ^e);

private:
  static EventWaitHandle ^s_evtServerLaunched = nullptr;
  static Action ^s_dltShutdownRequestCallback = nullptr;
  static Action<int> ^s_dltStatusReportCallback = nullptr;
  static ServiceHost ^s_host = nullptr;
  static int s_nServerObjectCount = 0;
  static Server ^s_svrPrimaryServerInstance = nullptr;

  int m_nClientInstance;
  String ^m_strSharedData;
  List<ClientMirrorImage ^> ^m_lstClientRegistry;
  Timers::Timer ^m_tmrHeartbeat;
  int m_nTimeout;
};

}
